@@ -35,7 +35,6 @@ extern crate lazy_static;
 extern crate log;
 #[macro_use]
 extern crate magic_crypt;
-extern crate rpassword;
 
 // External libs
 use argh::FromArgs;
@@ -46,8 +45,8 @@ use std::time::Duration;
 // Include
 mod activity_manager;
 mod config;
+mod explorer;
 mod filetransfer;
-mod fs;
 mod host;
 mod support;
 mod system;
@@ -57,28 +56,42 @@ mod utils;
 // namespaces
 use activity_manager::{ActivityManager, NextActivity};
 use filetransfer::FileTransferParams;
-use system::logging;
+use system::logging::{self, LogLevel};
 
 enum Task {
     Activity(NextActivity),
     ImportTheme(PathBuf),
+    InstallUpdate,
 }
 
 #[derive(FromArgs)]
 #[argh(description = "
-where positional can be: [protocol://user@address:port:wrkdir] [local-wrkdir]
+where positional can be: [address] [local-wrkdir]
+
+Address syntax can be:
+
+    - `protocol://user@address:port:wrkdir` for protocols such as Sftp, Scp, Ftp
+    - `s3://bucket-name@region:profile:/wrkdir` for Aws S3 protocol
 
 Please, report issues to <https://github.com/veeso/termscp>
-Please, consider supporting the author <https://www.buymeacoffee.com/veeso>")]
+Please, consider supporting the author <https://ko-fi.com/veeso>")]
 struct Args {
     #[argh(switch, short = 'c', description = "open termscp configuration")]
     config: bool,
+    #[argh(switch, short = 'D', description = "enable TRACE log level")]
+    debug: bool,
     #[argh(option, short = 'P', description = "provide password from CLI")]
     password: Option<String>,
     #[argh(switch, short = 'q', description = "disable logging")]
     quiet: bool,
     #[argh(option, short = 't', description = "import specified theme")]
     theme: Option<String>,
+    #[argh(
+        switch,
+        short = 'u',
+        description = "update termscp to the latest version"
+    )]
+    update: bool,
     #[argh(
         option,
         short = 'T',
@@ -99,7 +112,7 @@ struct Args {
 struct RunOpts {
     remote: Option<FileTransferParams>,
     ticks: Duration,
-    log_enabled: bool,
+    log_level: LogLevel,
     task: Task,
 }
 
@@ -108,7 +121,7 @@ impl Default for RunOpts {
         Self {
             remote: None,
             ticks: Duration::from_millis(10),
-            log_enabled: true,
+            log_level: LogLevel::Info,
             task: Task::Activity(NextActivity::Authentication),
         }
     }
@@ -125,10 +138,8 @@ fn main() {
         }
     };
     // Setup logging
-    if run_opts.log_enabled {
-        if let Err(err) = logging::init() {
-            eprintln!("Failed to initialize logging: {}", err);
-        }
+    if let Err(err) = logging::init(run_opts.log_level) {
+        eprintln!("Failed to initialize logging: {}", err);
     }
     // Read password from remote
     if let Err(err) = read_password(&mut run_opts) {
@@ -163,8 +174,10 @@ fn parse_args(args: Args) -> Result<RunOpts, String> {
         run_opts.task = Task::Activity(NextActivity::SetupActivity);
     }
     // Logging
-    if args.quiet {
-        run_opts.log_enabled = false;
+    if args.debug {
+        run_opts.log_level = LogLevel::Trace;
+    } else if args.quiet {
+        run_opts.log_level = LogLevel::Off;
     }
     // Match ticks
     run_opts.ticks = Duration::from_millis(args.ticks);
@@ -172,6 +185,9 @@ fn parse_args(args: Args) -> Result<RunOpts, String> {
     if let Some(theme) = args.theme {
         run_opts.task = Task::ImportTheme(PathBuf::from(theme));
     }
+    if args.update {
+        run_opts.task = Task::InstallUpdate;
+    }
     // @! Ordinary mode
     // Remote argument
     if let Some(remote) = args.positional.get(0) {
@@ -180,7 +196,9 @@ fn parse_args(args: Args) -> Result<RunOpts, String> {
             Ok(mut remote) => {
                 // If password is provided, set password
                 if let Some(passwd) = args.password {
-                    remote = remote.password(Some(passwd));
+                    if let Some(mut params) = remote.params.mut_generic_params() {
+                        params.password = Some(passwd);
+                    }
                 }
                 // Set params
                 run_opts.remote = Some(remote);
@@ -209,25 +227,28 @@ fn parse_args(args: Args) -> Result<RunOpts, String> {
 fn read_password(run_opts: &mut RunOpts) -> Result<(), String> {
     // Initialize client if necessary
     if let Some(remote) = run_opts.remote.as_mut() {
-        debug!("User has specified remote options: address: {:?}, port: {:?}, protocol: {:?}, user: {:?}, password: {}", remote.address, remote.port, remote.protocol, remote.username, utils::fmt::shadow_password(remote.password.as_deref().unwrap_or("")));
-        if remote.password.is_none() {
-            // Ask password if unspecified
-            remote.password = match rpassword::read_password_from_tty(Some("Password: ")) {
-                Ok(p) => {
-                    if p.is_empty() {
-                        None
-                    } else {
-                        debug!(
-                            "Read password from tty: {}",
-                            utils::fmt::shadow_password(p.as_str())
-                        );
-                        Some(p)
+        // Ask password for generic params
+        if let Some(mut params) = remote.params.mut_generic_params() {
+            // Ask password only if generic protocol params
+            if params.password.is_none() {
+                // Ask password if unspecified
+                params.password = match rpassword::read_password_from_tty(Some("Password: ")) {
+                    Ok(p) => {
+                        if p.is_empty() {
+                            None
+                        } else {
+                            debug!(
+                                "Read password from tty: {}",
+                                utils::fmt::shadow_password(p.as_str())
+                            );
+                            Some(p)
+                        }
                     }
-                }
-                Err(_) => {
-                    return Err("Could not read password from prompt".to_string());
-                }
-            };
+                    Err(_) => {
+                        return Err("Could not read password from prompt".to_string());
+                    }
+                };
+            }
         }
     }
     Ok(())
@@ -248,6 +269,16 @@ fn run(mut run_opts: RunOpts) -> i32 {
                 1
             }
         },
+        Task::InstallUpdate => match support::install_update() {
+            Ok(msg) => {
+                println!("{}", msg);
+                0
+            }
+            Err(err) => {
+                eprintln!("Could not install update: {}", err);
+                1
+            }
+        },
         Task::Activity(activity) => {
             // Get working directory
             let wrkdir: PathBuf = match env::current_dir() {
