@@ -28,18 +28,14 @@
 // Local
 use super::{KeyStorage, KeyStorageError};
 // Ext
-use keyring::{Keyring, KeyringError};
+use keyring::{Entry as Keyring, Error as KeyringError};
 
-/// ## KeyringStorage
-///
 /// provides a `KeyStorage` implementation using the keyring crate
 pub struct KeyringStorage {
     username: String,
 }
 
 impl KeyringStorage {
-    /// ### new
-    ///
     /// Instantiates a new KeyringStorage
     pub fn new(username: &str) -> Self {
         KeyringStorage {
@@ -49,8 +45,6 @@ impl KeyringStorage {
 }
 
 impl KeyStorage for KeyringStorage {
-    /// ### get_key
-    ///
     /// Retrieve key from the key storage.
     /// The key might be acccess through an identifier, which identifies
     /// the key in the storage
@@ -59,21 +53,17 @@ impl KeyStorage for KeyringStorage {
         match storage.get_password() {
             Ok(s) => Ok(s),
             Err(e) => match e {
-                KeyringError::NoPasswordFound => Err(KeyStorageError::NoSuchKey),
-                #[cfg(target_os = "windows")]
-                KeyringError::WindowsVaultError => Err(KeyStorageError::NoSuchKey),
-                #[cfg(target_os = "macos")]
-                KeyringError::MacOsKeychainError(_) => Err(KeyStorageError::NoSuchKey),
-                #[cfg(target_os = "linux")]
-                KeyringError::SecretServiceError(_) => Err(KeyStorageError::ProviderError),
-                KeyringError::Parse(_) => Err(KeyStorageError::BadSytax),
-                _ => Err(KeyStorageError::ProviderError),
+                KeyringError::NoEntry => Err(KeyStorageError::NoSuchKey),
+                KeyringError::PlatformFailure(_)
+                | KeyringError::NoStorageAccess(_)
+                | KeyringError::WrongCredentialPlatform => Err(KeyStorageError::ProviderError),
+                KeyringError::BadEncoding(_) | KeyringError::TooLong(_, _) => {
+                    Err(KeyStorageError::BadSytax)
+                }
             },
         }
     }
 
-    /// ### set_key
-    ///
     /// Set the key into the key storage
     fn set_key(&self, storage_id: &str, key: &str) -> Result<(), KeyStorageError> {
         let storage: Keyring = Keyring::new(storage_id, self.username.as_str());
@@ -92,13 +82,8 @@ impl KeyStorage for KeyringStorage {
         // Check what kind of error is returned
         match storage.get_password() {
             Ok(_) => true,
-            #[cfg(not(target_os = "linux"))]
-            Err(err) => !matches!(err, KeyringError::NoBackendFound),
-            #[cfg(target_os = "linux")]
-            Err(err) => !matches!(
-                err,
-                KeyringError::NoBackendFound | KeyringError::SecretServiceError(_)
-            ),
+            Err(KeyringError::NoStorageAccess(_) | KeyringError::PlatformFailure(_)) => false,
+            Err(_) => true,
         }
     }
 }
