@@ -26,10 +26,12 @@
  * SOFTWARE.
  */
 // modules
+pub mod auto_update;
 pub mod bookmarks_client;
 pub mod config_client;
 pub mod environment;
 pub(self) mod keys;
 pub mod logging;
+pub mod notifications;
 pub mod sshkey_storage;
 pub mod theme_provider;
