@@ -28,18 +28,46 @@
 // Locals
 use super::config_client::ConfigClient;
 // Ext
+use remotefs_ssh::SshKeyStorage as SshKeyStorageT;
 use std::collections::HashMap;
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
 pub struct SshKeyStorage {
     hosts: HashMap<String, PathBuf>, // Association between {user}@{host} and RSA key path
 }
 
 impl SshKeyStorage {
-    /// ### storage_from_config
-    ///
-    /// Create a `SshKeyStorage` starting from a `ConfigClient`
-    pub fn storage_from_config(cfg_client: &ConfigClient) -> Self {
+    /// Create an empty ssh key storage; used in case `ConfigClient` is not available
+    #[cfg(test)]
+    pub fn empty() -> Self {
+        SshKeyStorage {
+            hosts: HashMap::new(),
+        }
+    }
+
+    /// Make mapkey from host and username
+    fn make_mapkey(host: &str, username: &str) -> String {
+        format!("{}@{}", username, host)
+    }
+
+    #[cfg(test)]
+    /// Add a key to storage
+    /// NOTE: available only for tests
+    pub fn add_key(&mut self, host: &str, username: &str, p: PathBuf) {
+        let key: String = Self::make_mapkey(host, username);
+        self.hosts.insert(key, p);
+    }
+}
+
+impl SshKeyStorageT for SshKeyStorage {
+    fn resolve(&self, host: &str, username: &str) -> Option<&Path> {
+        let key: String = Self::make_mapkey(host, username);
+        self.hosts.get(&key).map(|x| x.as_path())
+    }
+}
+
+impl From<&ConfigClient> for SshKeyStorage {
+    fn from(cfg_client: &ConfigClient) -> Self {
         let mut hosts: HashMap<String, PathBuf> =
             HashMap::with_capacity(cfg_client.iter_ssh_keys().count());
         debug!("Setting up SSH key storage");
@@ -63,41 +91,6 @@ impl SshKeyStorage {
         // Return storage
         SshKeyStorage { hosts }
     }
-
-    /// ### empty
-    ///
-    /// Create an empty ssh key storage; used in case `ConfigClient` is not available
-    #[cfg(test)]
-    pub fn empty() -> Self {
-        SshKeyStorage {
-            hosts: HashMap::new(),
-        }
-    }
-
-    /// ### resolve
-    ///
-    /// Return RSA key path from host and username
-    pub fn resolve(&self, host: &str, username: &str) -> Option<&PathBuf> {
-        let key: String = Self::make_mapkey(host, username);
-        self.hosts.get(&key)
-    }
-
-    /// ### make_mapkey
-    ///
-    /// Make mapkey from host and username
-    fn make_mapkey(host: &str, username: &str) -> String {
-        format!("{}@{}", username, host)
-    }
-
-    #[cfg(test)]
-    /// ### add_key
-    ///
-    /// Add a key to storage
-    /// NOTE: available only for tests
-    pub fn add_key(&mut self, host: &str, username: &str, p: PathBuf) {
-        let key: String = Self::make_mapkey(host, username);
-        self.hosts.insert(key, p);
-    }
 }
 
 #[cfg(test)]
@@ -121,7 +114,7 @@ mod tests {
             .add_ssh_key("192.168.1.31", "pi", "piroporopero")
             .is_ok());
         // Create ssh key storage
-        let storage: SshKeyStorage = SshKeyStorage::storage_from_config(&client);
+        let storage: SshKeyStorage = SshKeyStorage::from(&client);
         // Verify key exists
         let mut exp_key_path: PathBuf = key_path.clone();
         exp_key_path.push("pi@192.168.1.31.key");
@@ -149,8 +142,6 @@ mod tests {
         );
     }
 
-    /// ### get_paths
-    ///
     /// Get paths for configuration and keys directory
     fn get_paths(dir: &Path) -> (PathBuf, PathBuf) {
         let mut k: PathBuf = PathBuf::from(dir);
