@@ -26,58 +26,52 @@
  * SOFTWARE.
  */
 // locals
-use super::{FileTransferActivity, FsEntry, LogLevel, SelectedEntry};
+use super::{FileTransferActivity, LogLevel, SelectedFile};
+
+use remotefs::File;
 
 impl FileTransferActivity {
     pub(crate) fn action_local_delete(&mut self) {
         match self.get_local_selected_entries() {
-            SelectedEntry::One(entry) => {
+            SelectedFile::One(entry) => {
                 // Delete file
                 self.local_remove_file(&entry);
-                // Reload
-                self.reload_local_dir();
             }
-            SelectedEntry::Many(entries) => {
+            SelectedFile::Many(entries) => {
                 // Iter files
                 for entry in entries.iter() {
                     // Delete file
                     self.local_remove_file(entry);
                 }
-                // Reload entries
-                self.reload_local_dir();
             }
-            SelectedEntry::None => {}
+            SelectedFile::None => {}
         }
     }
 
     pub(crate) fn action_remote_delete(&mut self) {
         match self.get_remote_selected_entries() {
-            SelectedEntry::One(entry) => {
+            SelectedFile::One(entry) => {
                 // Delete file
                 self.remote_remove_file(&entry);
-                // Reload
-                self.reload_remote_dir();
             }
-            SelectedEntry::Many(entries) => {
+            SelectedFile::Many(entries) => {
                 // Iter files
                 for entry in entries.iter() {
                     // Delete file
                     self.remote_remove_file(entry);
                 }
-                // Reload entries
-                self.reload_remote_dir();
             }
-            SelectedEntry::None => {}
+            SelectedFile::None => {}
         }
     }
 
-    pub(crate) fn local_remove_file(&mut self, entry: &FsEntry) {
+    pub(crate) fn local_remove_file(&mut self, entry: &File) {
         match self.host.remove(entry) {
             Ok(_) => {
                 // Log
                 self.log(
                     LogLevel::Info,
-                    format!("Removed file \"{}\"", entry.get_abs_path().display()),
+                    format!("Removed file \"{}\"", entry.path().display()),
                 );
             }
             Err(err) => {
@@ -85,7 +79,7 @@ impl FileTransferActivity {
                     LogLevel::Error,
                     format!(
                         "Could not delete file \"{}\": {}",
-                        entry.get_abs_path().display(),
+                        entry.path().display(),
                         err
                     ),
                 );
@@ -93,12 +87,12 @@ impl FileTransferActivity {
         }
     }
 
-    pub(crate) fn remote_remove_file(&mut self, entry: &FsEntry) {
-        match self.client.remove(entry) {
+    pub(crate) fn remote_remove_file(&mut self, entry: &File) {
+        match self.client.remove_dir_all(entry.path()) {
             Ok(_) => {
                 self.log(
                     LogLevel::Info,
-                    format!("Removed file \"{}\"", entry.get_abs_path().display()),
+                    format!("Removed file \"{}\"", entry.path().display()),
                 );
             }
             Err(err) => {
@@ -106,7 +100,7 @@ impl FileTransferActivity {
                     LogLevel::Error,
                     format!(
                         "Could not delete file \"{}\": {}",
-                        entry.get_abs_path().display(),
+                        entry.path().display(),
                         err
                     ),
                 );
