@@ -26,10 +26,10 @@
  * SOFTWARE.
  */
 // locals
-use super::{FileTransferActivity, FsEntry, LogLevel, SelectedEntry, TransferPayload};
-use crate::fs::FsFile;
+use super::{FileTransferActivity, LogLevel, SelectedFile, TransferPayload};
+
 // ext
-use crossterm::terminal::{disable_raw_mode, enable_raw_mode};
+use remotefs::File;
 use std::fs::OpenOptions;
 use std::io::Read;
 use std::path::{Path, PathBuf};
@@ -37,10 +37,10 @@ use std::time::SystemTime;
 
 impl FileTransferActivity {
     pub(crate) fn action_edit_local_file(&mut self) {
-        let entries: Vec<FsEntry> = match self.get_local_selected_entries() {
-            SelectedEntry::One(entry) => vec![entry],
-            SelectedEntry::Many(entries) => entries,
-            SelectedEntry::None => vec![],
+        let entries: Vec<File> = match self.get_local_selected_entries() {
+            SelectedFile::One(entry) => vec![entry],
+            SelectedFile::Many(entries) => entries,
+            SelectedFile::None => vec![],
         };
         // Edit all entries
         for entry in entries.iter() {
@@ -48,44 +48,38 @@ impl FileTransferActivity {
             if entry.is_file() {
                 self.log(
                     LogLevel::Info,
-                    format!("Opening file \"{}\"…", entry.get_abs_path().display()),
+                    format!("Opening file \"{}\"…", entry.path().display()),
                 );
                 // Edit file
-                if let Err(err) = self.edit_local_file(entry.get_abs_path().as_path()) {
+                if let Err(err) = self.edit_local_file(entry.path()) {
                     self.log_and_alert(LogLevel::Error, err);
                 }
             }
         }
-        // Reload entries
-        self.reload_local_dir();
     }
 
     pub(crate) fn action_edit_remote_file(&mut self) {
-        let entries: Vec<FsEntry> = match self.get_remote_selected_entries() {
-            SelectedEntry::One(entry) => vec![entry],
-            SelectedEntry::Many(entries) => entries,
-            SelectedEntry::None => vec![],
+        let entries: Vec<File> = match self.get_remote_selected_entries() {
+            SelectedFile::One(entry) => vec![entry],
+            SelectedFile::Many(entries) => entries,
+            SelectedFile::None => vec![],
         };
         // Edit all entries
         for entry in entries.into_iter() {
             // Check if file
-            if let FsEntry::File(file) = entry {
+            if entry.is_file() {
                 self.log(
                     LogLevel::Info,
-                    format!("Opening file \"{}\"…", file.abs_path.display()),
+                    format!("Opening file \"{}\"…", entry.path().display()),
                 );
                 // Edit file
-                if let Err(err) = self.edit_remote_file(file) {
+                if let Err(err) = self.edit_remote_file(entry) {
                     self.log_and_alert(LogLevel::Error, err);
                 }
             }
         }
-        // Reload entries
-        self.reload_remote_dir();
     }
 
-    /// ### edit_local_file
-    ///
     /// Edit a file on localhost
     fn edit_local_file(&mut self, path: &Path) -> Result<(), String> {
         // Read first 2048 bytes or less from file to check if it is textual
@@ -109,14 +103,15 @@ impl FileTransferActivity {
             }
         }
         // Put input mode back to normal
-        if let Err(err) = disable_raw_mode() {
+        if let Err(err) = self.context_mut().terminal().disable_raw_mode() {
             error!("Failed to disable raw mode: {}", err);
         }
         // Leave alternate mode
-        #[cfg(not(target_os = "windows"))]
-        if let Some(ctx) = self.context.as_mut() {
-            ctx.leave_alternate_screen();
+        if let Err(err) = self.context_mut().terminal().leave_alternate_screen() {
+            error!("Could not leave alternate screen: {}", err);
         }
+        // Lock ports
+        assert!(self.app.lock_ports().is_ok());
         // Open editor
         match edit::edit_file(path) {
             Ok(_) => self.log(
@@ -128,30 +123,35 @@ impl FileTransferActivity {
             ),
             Err(err) => return Err(format!("Could not open editor: {}", err)),
         }
-        #[cfg(not(target_os = "windows"))]
         if let Some(ctx) = self.context.as_mut() {
-            // Clear screen
-            ctx.clear_screen();
             // Enter alternate mode
-            ctx.enter_alternate_screen();
+            if let Err(err) = ctx.terminal().enter_alternate_screen() {
+                error!("Could not enter alternate screen: {}", err);
+            }
+            // Re-enable raw mode
+            if let Err(err) = ctx.terminal().enable_raw_mode() {
+                error!("Failed to enter raw mode: {}", err);
+            }
+            // Clear screens
+            if let Err(err) = ctx.terminal().clear_screen() {
+                error!("Could not clear screen screen: {}", err);
+            }
+            // Unlock ports
+            assert!(self.app.unlock_ports().is_ok());
         }
-        // Re-enable raw mode
-        let _ = enable_raw_mode();
         Ok(())
     }
 
-    /// ### edit_remote_file
-    ///
     /// Edit file on remote host
-    fn edit_remote_file(&mut self, file: FsFile) -> Result<(), String> {
+    fn edit_remote_file(&mut self, file: File) -> Result<(), String> {
         // Create temp file
         let tmpfile: PathBuf = match self.download_file_as_temp(&file) {
             Ok(p) => p,
             Err(err) => return Err(err),
         };
         // Download file
-        let file_name = file.name.clone();
-        let file_path = file.abs_path.clone();
+        let file_name = file.name();
+        let file_path = file.path().to_path_buf();
         if let Err(err) = self.filetransfer_recv(
             TransferPayload::File(file),
             tmpfile.as_path(),
@@ -161,7 +161,7 @@ impl FileTransferActivity {
         }
         // Get current file modification time
         let prev_mtime: SystemTime = match self.host.stat(tmpfile.as_path()) {
-            Ok(e) => e.get_last_change_time(),
+            Ok(e) => e.metadata().modified.unwrap_or(std::time::UNIX_EPOCH),
             Err(err) => {
                 return Err(format!(
                     "Could not stat \"{}\": {}",
@@ -175,7 +175,7 @@ impl FileTransferActivity {
             return Err(err);
         }
         // Get local fs entry
-        let tmpfile_entry: FsEntry = match self.host.stat(tmpfile.as_path()) {
+        let tmpfile_entry: File = match self.host.stat(tmpfile.as_path()) {
             Ok(e) => e,
             Err(err) => {
                 return Err(format!(
@@ -186,7 +186,12 @@ impl FileTransferActivity {
             }
         };
         // Check if file has changed
-        match prev_mtime != tmpfile_entry.get_last_change_time() {
+        match prev_mtime
+            != tmpfile_entry
+                .metadata()
+                .modified
+                .unwrap_or(std::time::UNIX_EPOCH)
+        {
             true => {
                 self.log(
                     LogLevel::Info,
@@ -196,8 +201,8 @@ impl FileTransferActivity {
                     ),
                 );
                 // Get local fs entry
-                let tmpfile_entry: FsFile = match self.host.stat(tmpfile.as_path()) {
-                    Ok(e) => e.unwrap_file(),
+                let tmpfile_entry = match self.host.stat(tmpfile.as_path()) {
+                    Ok(e) => e,
                     Err(err) => {
                         return Err(format!(
                             "Could not stat \"{}\": {}",
