@@ -29,22 +29,20 @@
 // Locals
 use super::SetupActivity;
 // Ext
-use crossterm::terminal::{disable_raw_mode, enable_raw_mode};
 use std::env;
 
 impl SetupActivity {
-    /// ### save_config
-    ///
     /// Save configuration
     pub(super) fn save_config(&mut self) -> Result<(), String> {
         match self.config().write_config() {
             Ok(_) => Ok(()),
-            Err(err) => Err(format!("Could not save configuration: {}", err)),
+            Err(err) => {
+                error!("Could not save configuration: {}", err);
+                Err(format!("Could not save configuration: {}", err))
+            }
         }
     }
 
-    /// ### reset_config_changes
-    ///
     /// Reset configuration changes; pratically read config from file, overwriting any change made
     /// since last write action
     pub(super) fn reset_config_changes(&mut self) -> Result<(), String> {
@@ -53,8 +51,6 @@ impl SetupActivity {
             .map_err(|e| format!("Could not reload configuration: {}", e))
     }
 
-    /// ### save_theme
-    ///
     /// Save theme to file
     pub(super) fn save_theme(&mut self) -> Result<(), String> {
         self.theme_provider()
@@ -62,8 +58,6 @@ impl SetupActivity {
             .map_err(|e| format!("Could not save theme: {}", e))
     }
 
-    /// ### reset_theme_changes
-    ///
     /// Reset changes committed to theme
     pub(super) fn reset_theme_changes(&mut self) -> Result<(), String> {
         self.theme_provider()
@@ -71,8 +65,6 @@ impl SetupActivity {
             .map_err(|e| format!("Could not restore theme: {}", e))
     }
 
-    /// ### delete_ssh_key
-    ///
     /// Delete ssh key from config cli
     pub(super) fn delete_ssh_key(&mut self, host: &str, username: &str) -> Result<(), String> {
         match self.config_mut().del_ssh_key(host, username) {
@@ -84,8 +76,6 @@ impl SetupActivity {
         }
     }
 
-    /// ### edit_ssh_key
-    ///
     /// Edit selected ssh key
     pub(super) fn edit_ssh_key(&mut self, idx: usize) -> Result<(), String> {
         match self.context.as_mut() {
@@ -94,12 +84,15 @@ impl SetupActivity {
                 // Set editor if config client exists
                 env::set_var("EDITOR", ctx.config().get_text_editor());
                 // Prepare terminal
-                if let Err(err) = disable_raw_mode() {
+                if let Err(err) = ctx.terminal().disable_raw_mode() {
                     error!("Failed to disable raw mode: {}", err);
                 }
                 // Leave alternate mode
-                #[cfg(not(target_os = "windows"))]
-                ctx.leave_alternate_screen();
+                if let Err(err) = ctx.terminal().leave_alternate_screen() {
+                    error!("Could not leave alternate screen: {}", err);
+                }
+                // Lock ports
+                assert!(self.app.lock_ports().is_ok());
                 // Get result
                 let result: Result<(), String> = match ctx.config().iter_ssh_keys().nth(idx) {
                     Some(key) => {
@@ -121,22 +114,25 @@ impl SetupActivity {
                 };
                 // Restore terminal
                 // Clear screen
-                ctx.clear_screen();
+                if let Err(err) = ctx.terminal().clear_screen() {
+                    error!("Could not clear screen screen: {}", err);
+                }
                 // Enter alternate mode
-                #[cfg(not(target_os = "windows"))]
-                ctx.enter_alternate_screen();
+                if let Err(err) = ctx.terminal().enter_alternate_screen() {
+                    error!("Could not enter alternate screen: {}", err);
+                }
                 // Re-enable raw mode
-                if let Err(err) = enable_raw_mode() {
+                if let Err(err) = ctx.terminal().enable_raw_mode() {
                     error!("Failed to enter raw mode: {}", err);
                 }
+                // Unlock ports
+                assert!(self.app.unlock_ports().is_ok());
                 // Return result
                 result
             }
         }
     }
 
-    /// ### add_ssh_key
-    ///
     /// Add provided ssh key to config client
     pub(super) fn add_ssh_key(
         &mut self,
