@@ -25,7 +25,7 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-use crate::fs::UnixPex;
+use remotefs::fs::UnixPexClass;
 
 use chrono::prelude::*;
 use std::path::{Path, PathBuf};
@@ -35,18 +35,18 @@ use tuirealm::tui::style::Color;
 /// ### fmt_pex
 ///
 /// Convert permissions bytes of permissions value into ls notation (e.g. rwx,-wx,--x)
-pub fn fmt_pex(pex: UnixPex) -> String {
+pub fn fmt_pex(pex: UnixPexClass) -> String {
     format!(
         "{}{}{}",
-        match pex.can_read() {
+        match pex.read() {
             true => 'r',
             false => '-',
         },
-        match pex.can_write() {
+        match pex.write() {
             true => 'w',
             false => '-',
         },
-        match pex.can_execute() {
+        match pex.execute() {
             true => 'x',
             false => '-',
         }
@@ -287,6 +287,25 @@ pub fn shadow_password(s: &str) -> String {
     (0..s.len()).map(|_| '*').collect()
 }
 
+/// ### fmt_bytes
+///
+/// Format bytes
+pub fn fmt_bytes(v: u64) -> String {
+    if v >= 1125899906842624 {
+        format!("{} PB", v / 1125899906842624)
+    } else if v >= 1099511627776 {
+        format!("{} TB", v / 1099511627776)
+    } else if v >= 1073741824 {
+        format!("{} GB", v / 1073741824)
+    } else if v >= 1048576 {
+        format!("{} MB", v / 1048576)
+    } else if v >= 1024 {
+        format!("{} KB", v / 1024)
+    } else {
+        format!("{} B", v)
+    }
+}
+
 #[cfg(test)]
 mod tests {
 
@@ -296,9 +315,9 @@ mod tests {
 
     #[test]
     fn test_utils_fmt_pex() {
-        assert_eq!(fmt_pex(UnixPex::from(7)), String::from("rwx"));
-        assert_eq!(fmt_pex(UnixPex::from(5)), String::from("r-x"));
-        assert_eq!(fmt_pex(UnixPex::from(6)), String::from("rw-"));
+        assert_eq!(fmt_pex(UnixPexClass::from(7)), String::from("rwx"));
+        assert_eq!(fmt_pex(UnixPexClass::from(5)), String::from("r-x"));
+        assert_eq!(fmt_pex(UnixPexClass::from(6)), String::from("rw-"));
     }
 
     #[test]
@@ -599,4 +618,14 @@ mod tests {
     fn test_utils_fmt_shadow_password() {
         assert_eq!(shadow_password("foobar"), String::from("******"));
     }
+
+    #[test]
+    fn format_bytes() {
+        assert_eq!(fmt_bytes(110).as_str(), "110 B");
+        assert_eq!(fmt_bytes(2048).as_str(), "2 KB");
+        assert_eq!(fmt_bytes(2097152).as_str(), "2 MB");
+        assert_eq!(fmt_bytes(4294967296).as_str(), "4 GB");
+        assert_eq!(fmt_bytes(3298534883328).as_str(), "3 TB");
+        assert_eq!(fmt_bytes(3377699720527872).as_str(), "3 PB");
+    }
 }
